-include ../site.mk

CODENAME=fluorine

# Where source rpms are. It can be a list of directories.
ifndef SRPMS-DIR
SRPMS-DIR=../../SRPMS
endif

# Where we build and store the rpms.
ifndef RPMTOPDIR
RPMTOPDIR=/export/rpms
endif

BUILD-ARCH=$(shell arch)

ifneq (,$(findstring 86, $(BUILD-ARCH)))
BUILD-ARCH=i386
endif

ifeq (athlon,$(BUILD-ARCH))
BUILD-ARCH=i386
endif

ifeq (k6,$(BUILD-ARCH))
BUILD-ARCH=i386
endif

ifndef RPM-CPU
RPM-CPU=mipsel
endif

ifndef RPM-ARCH
RPM-ARCH=$(RPM-CPU)
endif

ifneq (,$(findstring 86, $(RPM-ARCH)))
RPM-ARCH=i386
endif

ifeq (athlon,$(RPM-ARCH))
RPM-ARCH=i386
endif

ifeq (k6,$(RPM-ARCH))
RPM-ARCH=i386
endif

HOSTNAME=$(shell hostname)
ifeq (,$(findstring lucon.org, $(HOSTNAME)))
BASELINE-BUILD?=true
else
BASELINE-BUILD?=false
endif

ifeq (true,$(BASELINE-BUILD))
BUILD-MIPS2?=true
else
BUILD-MIPS2?=false
endif

ifeq (true,$(BUILD-MIPS2))
MIPS-CFLAGS=-mips2
mips-RPMFLAGS=\
	--with mips2 \
	--define 'mips_isa $(MIPS-CFLAGS)'
mipsel-RPMFLAGS=$(mips-RPMFLAGS)
NATIVE-mipsel-RPMFLAGS=$(mipsel-RPMFLAGS)
NATIVE-mips-RPMFLAGS=$(NATIVE-mipsel-RPMFLAGS)
endif

ifdef VENDOR
VENDOR-TARGET=$(RPM-CPU)-$(VENDOR)
TOOLTARGET=$(RPM-ARCH)-$(VENDOR)-linux
else
VENDOR-TARGET=$(RPM-CPU)
TOOLTARGET=$(RPM-ARCH)-linux
endif

TARGET=$(VENDOR-TARGET)-linux

ifndef RPMTMPDIR
RPMTMPDIR=/var/tmp/$(TARGET)
endif

# Where the toolchain is.
ifdef TOOLPREFIX
TOOLPREFIX:=$(subst $(VENDOR-TARGET),$(VENDOR-TARGET)-$(CODENAME),$(TOOLPREFIX))
else
TOOLPREFIX=/export/tools
endif

PATH:=$(TOOLPREFIX)/bin:$(PATH)
export PATH

TOOLDIR=$(TOOLPREFIX)/$(TOOLTARGET)

ifndef FINDRPM
FINDRPM=findrpm
endif

GENERATED+=../rpmrc ../rpmrc.native

ifndef SPEC
SPEC=$(NAME).spec
endif

RPMDIRS=$(addprefix $(RPMTOPDIR)/, BUILD RPMS SOURCES SPECS SRPMS)
RPMDIRS+=$(RPMTOPDIR)/RPMS/noarch $(RPMTOPDIR)/RPMS/$(RPM-CPU)
ifneq ($(RPM-CPU),$(BUILD-ARCH))
RPMDIRS+=$(RPMTOPDIR)/RPMS/$(BUILD-ARCH)
endif

RPMFLAGS+=\
	--with crosscompile \
	--define '_arch $(RPM-ARCH)' \
	--define '_topdir $(RPMTOPDIR)' \
	--define '_tmppath $(RPMTMPDIR)' \

NATIVE-RPMFLAGS+=\
	--define '_topdir $(RPMTOPDIR)' \
	--define '_tmppath $(RPMTMPDIR)' \

ifdef VENDOR
RPMFLAGS+=\
	--define '_vendor $(VENDOR)' \
	--define '_target_prefix $(RPM-ARCH)-$(VENDOR)'
else
RPMFLAGS+=\
	--define '_target_prefix $(RPM-ARCH)'
endif

RPMFLAGS+=$($(RPM-CPU)-RPMFLAGS)
NATIVE-RPMFLAGS+=$(NATIVE-$(RPM-CPU)-RPMFLAGS)

RPM=rpm
RPMBUILD=rpmbuild
RPM2CPIO=rpm2cpio

# Tell make where to find the source rpm.
vpath %.src.rpm $(SRPMS-DIR)

# Tell make where to find the tar files
vpath %.tar.gz $(SRPMS-DIR)
vpath %.tar.bz2 $(SRPMS-DIR)

ifndef DEFAULT-TARGET
DEFAULT-TARGET=rpm
endif

DEFAULT-TARGET: $(DEFAULT-TARGET)

rpm: $(RPMDIRS) DONE.$(RPM-CPU).rpm

ifdef PACK-PATCHES
SOURCES=$(PACK-PATCHES)
else
SOURCES=$(PATCHES)

ifdef RELEASE 
SOURCES+=$(NAME)-$(VERSION)-$(RELEASE).src.rpm
endif
endif

NO_SPEC=no

ifneq (,$(findstring $(SPEC), $(GENERATED)))
NO_SPEC=yes
endif

ifeq (,$(wildcard $(SPEC)))
NO_SPEC=yes
endif

ifeq (yes,$(NO_SPEC))

source-rpm=$(NAME)-$(VERSION)-$(RELEASE).src.rpm
found-source-rpm=\
  $(foreach dir,$(SRPMS-DIR), \
	    $(wildcard $(addprefix $(dir)/,$(source-rpm))))
ifneq (,$(findstring src.rpm, $(found-source-rpm)))
REBUILD=true
endif

ifndef REBUILD
source-tar=$(NAME)-$(VERSION).tar.gz
ifeq (,$(findstring $(source-tar), $(PACK-PATCHES)))
found-source-tar=\
  $(foreach dir,$(SRPMS-DIR), \
	    $(wildcard $(addprefix $(dir)/,$(source-tar))))
ifneq (,$(findstring tar.gz, $(found-source-tar)))
REBUILD=true
TAR-FILE=$(source-tar)
endif
endif
endif

ifndef REBUILD
ifdef PACK-PATCHES
$(NAME)-$(VERSION)-$(RELEASE).src.rpm: $(RPMDIRS) unpack $(SPEC)
	time $(RPMBUILD) --target=$(RPM-CPU) $(RPMFLAGS) \
		--rcfile ../rpmrc --short-circuit -bs $(SPEC)
	cp -af $(RPMTOPDIR)/SRPMS/$@ .
endif
endif

endif

complete modified:
	for cpu in $(RPM-CPU) $(COMPLETE-CPU-$(RPM-CPU)); do \
	  $(MAKE) native RPM-CPU=$$cpu \
		NATIVE-TARGET-RPMFLAGS="--target=$$cpu" || exit 1; \
	done

ifneq (,$(TAR-FILE))
$(NAME)-$(VERSION)-$(RELEASE).src.rpm: ../rpmrc $(RPMDIRS) $(TAR-FILE)
	$(RPMBUILD) --rcfile ../rpmrc $(RPMFLAGS) \
		--short-circuit -ts $(filter %.tar.gz, $^)
	cp -af $(RPMTOPDIR)/SRPMS/$@ .
endif

ifeq (true,$(REBUILD))
DONE.$(RPM-CPU).rpm: ../rpmrc $(NAME)-$(VERSION)-$(RELEASE).src.rpm
	[ -e $@ ] \
	|| time $(RPMBUILD) --target=$(RPM-CPU) $(RPMFLAGS) \
		--rcfile ../rpmrc --rebuild $(filter %.src.rpm, $^)
	touch $@

native: $(RPMDIRS) DONE.$(RPM-CPU).native

DONE.$(RPM-CPU).native: ../rpmrc.native $(NAME)-$(VERSION)-$(RELEASE).src.rpm
	[ -e $@ ] \
	|| time $(RPMBUILD) $(NATIVE-TARGET-RPMFLAGS) \
		$(NATIVE-RPMFLAGS) --rcfile ../rpmrc.native \
		--rebuild $(filter %.src.rpm, $^)
	touch $@

prepare: ../rpmrc $(NAME)-$(VERSION)-$(RELEASE).src.rpm
	time $(RPM) $(RPMFLAGS) \
		--rcfile ../rpmrc -U $(filter %.src.rpm, $^)
	time $(RPMBUILD) --target=$(RPM-CPU) $(RPMFLAGS) \
		--rcfile ../rpmrc -bp $(RPMTOPDIR)/SPECS/$(SPEC)

patch: ../rpmrc $(NAME)-$(VERSION)-$(RELEASE).src.rpm
	time $(RPM) $(RPMFLAGS) \
		--rcfile ../rpmrc -U $(filter %.src.rpm, $^)
	sed -e "s/^%patch\(.*\) -b .*$$/%patch\1/" \
		$(RPMTOPDIR)/SPECS/$(SPEC) > $(SPEC).patch
	time $(RPMBUILD) --target=$(RPM-CPU) $(RPMFLAGS) \
		--rcfile ../rpmrc -bp $(SPEC).patch
	rm -f $(SPEC).patch
	d=$(RPMTOPDIR)/BUILD/$(NAME)-$(VERSION); \
	if [ ! -d "$$d" ]; then \
	  for d in `ls -d $(RPMTOPDIR)/BUILD/$(NAME)-$(VERSION)*`; do \
	    break; \
	  done; \
	  if [ ! -d "$$d" ]; then \
	    d=$(RPMTOPDIR)/BUILD/$(NAME); \
	    if [ ! -d "$$d" ]; then \
	      for d in `ls -d $(RPMTOPDIR)/BUILD/$(NAME)*`; do \
		break; \
	      done; \
	    fi; \
	  fi; \
	fi; \
	if [ -d "$$d" ]; then \
	  find $$d -name "*.orig" -o -name "*.patch" -o -name CVS | xargs rm -rf; \
	fi
else
DONE.$(RPM-CPU).rpm: unpack $(SPEC)
	[ -e $@ ] \
	|| time $(RPMBUILD) --target=$(RPM-CPU) $(RPMFLAGS) \
		--rcfile ../rpmrc -ba $(SPEC)
	touch $@

native: $(RPMDIRS) DONE.$(RPM-CPU).native.rpm

DONE.$(RPM-CPU).native.rpm: ../rpmrc.native unpack $(SPEC)
	[ -e $@ ] \
	|| time $(RPMBUILD) $(NATIVE-TARGET-RPMFLAGS) \
		$(NATIVE-RPMFLAGS) \
		--rcfile ../rpmrc.native -ba $(SPEC)
	touch $@

prepare: $(RPMDIRS) unpack $(SPEC)
	time $(RPMBUILD) --target=$(RPM-CPU) $(RPMFLAGS) \
		--rcfile ../rpmrc -bp $(SPEC)

patch: $(RPMDIRS) unpack $(SPEC)
	sed -e "s/^%patch\(.*\) -b .*$$/%patch\1/" \
		$(SPEC) > $(SPEC).patch
	time $(RPMBUILD) --target=$(RPM-CPU) $(RPMFLAGS) \
		--rcfile ../rpmrc -bp $(SPEC).patch
	rm -f $(SPEC).patch
	d=$(RPMTOPDIR)/BUILD/$(NAME)-$(VERSION); \
	if [ ! -d "$$d" ]; then \
	  for d in `ls -d $(RPMTOPDIR)/BUILD/$(NAME)-$(VERSION)*`; do \
	    break; \
	  done; \
	  if [ ! -d "$$d" ]; then \
	    d=$(RPMTOPDIR)/BUILD/$(NAME); \
	    if [ ! -d "$$d" ]; then \
	      for d in `ls -d $(RPMTOPDIR)/BUILD/$(NAME)*`; do \
		break; \
	      done; \
	    fi; \
	  fi; \
	fi; \
	if [ -d "$$d" ]; then \
	  find $$d -name "*.orig" -o -name "*.patch" -o -name CVS | xargs rm -rf; \
	fi

unpack: sources

sources: ../rpmrc $(SOURCES)
	if [ -n "$(filter %.src.rpm, $^)" ]; then \
	  for r in $(filter %.src.rpm, $^); do \
	    $(RPM) --rcfile ../rpmrc $(RPMFLAGS) -Uvh $$r || exit 1; \
	  done; \
	fi
	if [ -n "$(filter-out ../rpmrc %.src.rpm, $^)" ]; then \
	  cp -afL $(filter-out ../rpmrc %.src.rpm, $^) \
		$(RPMTOPDIR)/SOURCES; \
	fi
	if [ -n "$(REPLACEMENTS)" ]; then \
	  for f in $(REPLACEMENTS); do \
	    chmod +w $(RPMTOPDIR)/SOURCES/$$f; \
	  done; \
	fi
endif

srpm: $(NAME)-$(VERSION)-$(RELEASE).src.rpm

$(RPMDIRS):
	mkdir -p $@

../rpmrc: ../rpmrc.native
	PWD=`pwd`; \
	TOP_DIR=`dirname $$PWD`; \
	sed -e "s,\(^macrofiles:.*\$\),\1:$$TOP_DIR/rpmmacros," \
		$^ > $@

../rpmrc.native: /usr/lib/rpm/rpmrc
	sed -e "s/mips[ \t].*-O2.*/mips -O2 $(MIPS-CFLAGS)/" \
	    -e "s/mipsel[ \t].*-O2.*/mipsel -O2 $(MIPS-CFLAGS)/" \
		$^ > $@

clobber:
	rm -fr $(GENERATED) *.src.rpm
	for cpu in $(RPM-CPU) $(COMPLETE-CPU-$(RPM-CPU)); do \
	  rm -rf DONE.$$cpu.*; \
	done

install::
ifneq (,$(TOOLS))
	rm -rf tools-$(RPM-CPU)
	mkdir tools-$(RPM-CPU)
	mkdir -p $(TOOLDIR)
	tool_dir=$$($(TARGET)-gcc --print-file-name=crt1.o) || exit 1; \
	tool_dir=$$(dirname $$tool_dir)/..; \
	cd tools-$(RPM-CPU); \
	for t in $(TOOLS); do \
	  file=`$(FINDRPM) $(RPMTOPDIR)/RPMS/$(RPM-CPU) $$t`; \
	  if test -z "$$file"; then \
	    echo "The $$t rpm is missing!"; \
	    exit 1; \
	  fi; \
	  $(RPM2CPIO) $$file | cpio -idm; \
	  find -type f | xargs chmod +r; \
	  find -type d | xargs chmod +rx; \
	  [ -d usr/lib/pkgconfig ] \
	  && find usr/lib/pkgconfig -name *.pc \
	     | xargs perl -p -i -e "s,=/usr,=$$tool_dir,g"; \
	  for d in usr/include usr/lib; do \
	    if [ -d $$d ]; then \
	      cp -avf $$d $(TOOLDIR); \
	    fi; \
	  done; \
	done
	rm -rf tools-$(RPM-CPU)
	find $(TOOLDIR)/lib -name lib*.la | xargs rm -f
else
	@true
endif

version:
	@file=`$(FINDRPM) $(RPMTOPDIR)/RPMS/$(RPM-CPU) $(PACKAGE)`; \
	[ -n "$$file" ] || file=`$(FINDRPM) $(RPMTOPDIR)/RPMS/noarch $(PACKAGE)`; \
	if [ -n "$$file" ]; then \
	  $(RPM) -qp $$file; \
	else \
	  echo $(PACKAGE) does not exist.; \
	fi

i386 i586 mips mipsel:
	$(MAKE) RPM-CPU=$@

clobber.%:
	$(MAKE) RPM-CPU=$(subst clobber.,,$@) clobber

install.%:
	$(MAKE) RPM-CPU=$(subst install.,,$@) install

# Check if we are root.
root:
	@if [ `id -u` != "0" ]; then \
	  echo You must be root to run this in this directory.; \
	  false; \
	else true; fi
